package com.huimv.eartag2.manage.service.impl;

import com.huimv.eartag2.common.dao.entity.EartagDeviceRegisterEntity;
import com.huimv.eartag2.common.dao.repo.EartagDeviceRegisterRepo;
import com.huimv.eartag2.common.utils.BizConst;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.manage.service.ICacheService;
import com.huimv.eartag2.manage.service.IDeviceRegisterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceRegisterServiceImpl implements IDeviceRegisterService {
    @Autowired
    private EartagDeviceRegisterRepo deviceRegisterRepo;
    @Autowired
    private ICacheService cacheService;

    @Override
    public Result addDevice(String chipId, String deviceCode, Integer deviceStatus, Integer activeStatus, String deviceAlias, String location, String remark, String farmId) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        Date todayDate = dateUtil.getTodayDate();
        Timestamp nowTimestamp = dateUtil.getTimestamp();
        //判断是否存在相同的设备编码
        if(isExistSameDevice(deviceCode)){
            return new Result(BizConst.CODE_DEVICE_CODE_EXIST,BizConst.MSG_DEVICE_CODE_EXIST,false);
        }
        //
        EartagDeviceRegisterEntity deviceRegisterEntity = new EartagDeviceRegisterEntity();
        deviceRegisterEntity.setChipId(chipId.trim());
        deviceRegisterEntity.setDeviceCode(deviceCode.trim());
        deviceRegisterEntity.setFarmId(farmId.trim());
        deviceRegisterEntity.setRegisterTime(nowTimestamp);
        deviceRegisterEntity.setActiveStatus(activeStatus);
        deviceRegisterEntity.setDeviceStatus(deviceStatus);
        deviceRegisterEntity.setDeviceAlias(deviceAlias.trim());
        deviceRegisterEntity.setLocation(location.trim());
        deviceRegisterEntity.setRemark(remark.trim());
        deviceRegisterEntity.setCreateDate(new java.sql.Date(todayDate.getTime()));
        deviceRegisterRepo.saveAndFlush(deviceRegisterEntity);
        //加入缓存
        cacheService.putAllDeviceSet(deviceCode);
        //加入缓存
        cacheService.putChipIdToCache(chipId,deviceCode);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result editDevice(String chipId, String deviceCode, Integer deviceStatus, Integer activeStatus, String deviceAlias, String location, String remark, Integer id) {
        //
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getById(id);
//        deviceRegisterEntity.setChipId(chipId);
//        deviceRegisterEntity.setDeviceCode(deviceCode);
        deviceRegisterEntity.setDeviceStatus(deviceStatus);
        deviceRegisterEntity.setActiveStatus(activeStatus);
        deviceRegisterEntity.setDeviceAlias(deviceAlias);
        deviceRegisterEntity.setLocation(location);
        deviceRegisterEntity.setRemark(remark);
        deviceRegisterRepo.saveAndFlush(deviceRegisterEntity);
        return new Result(ResultCode.SUCCESS);
    }

//    @Override
//    public Result removeDevice(String ids) {
//        String[] idsArray = ids.split(",");
//        List<Integer> idList = new ArrayList();
////        for(int a=0;a<idsArray.length;a++){
////            idList.add(Integer.parseInt(idsArray[a]));
////        }
////        deviceRegisterRepo.removeDevice(idList);
////        List<EartagDeviceRegisterEntity> deviceRegisterEntityList = deviceRegisterRepo.getDeviceRegisterByIds(idList);
////        deviceRegisterRepo.deleteAll(deviceRegisterEntityList);
////        deviceRegisterRepo.removeDevice();
//
////        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getDeviceRegister(ids);
//        return new Result(ResultCode.SUCCESS);
//    }

    @Override
    public Result listDevice(String farmId, String deviceCode, Integer deviceStatus, Integer activeStatus, String deviceAlias, String location, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagDeviceRegisterEntity> sf = (Specification<EartagDeviceRegisterEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //farmId
            if (null != farmId && farmId.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(String.class), farmId));
            }
            //deviceCode
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            //createDate
//            if (null != createDate && createDate.trim().length()>0) {
//                try {
//                    predList.add(criteriaBuilder.equal(root.get("createDate").as(java.sql.Date.class), new java.sql.Date(new DateUtil().parseDateTextToLong(createDate))));
//                } catch (ParseException e) {
//                    e.printStackTrace();
//                }
//            }
            //deviceStatus
            if (null != deviceStatus) {
                predList.add(criteriaBuilder.equal(root.get("deviceStatus").as(Integer.class), deviceStatus));
            }
            //activeStatus
            if (null != activeStatus) {
                predList.add(criteriaBuilder.equal(root.get("activeStatus").as(Integer.class), activeStatus));
            }
            //location
            if (null != location && location.trim().length()>0) {
                predList.add(criteriaBuilder.like(root.get("location").as(String.class), location));
            }
            //deviceAlias
            if (null != deviceAlias && deviceAlias.trim().length()>0) {
                predList.add(criteriaBuilder.like(root.get("deviceAlias").as(String.class), deviceAlias));
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        Page<EartagDeviceRegisterEntity> entityPage = deviceRegisterRepo.findAll(sf, pageable);
        return new Result(ResultCode.SUCCESS, entityPage);
    }

    @Override
    public Result removeBatchDevice(String ids) {
        String id = "1";
        Integer id2 = 1;

        deviceRegisterRepo.getById(id2);

        return null;
    }

    @Override
    public Result cancelDevice(Integer id, Integer deviceStatus) {
        //
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getById(id);
        if(deviceRegisterEntity == null){
            log.error(BizConst.MSG_DEVICE_REGISTER_NO_EXIST);
            return new Result(BizConst.CODE_DEVICE_REGISTER_NO_EXIST,BizConst.MSG_DEVICE_REGISTER_NO_EXIST,false);
        }
        deviceRegisterEntity.setDeviceStatus(deviceStatus);
        deviceRegisterRepo.saveAndFlush(deviceRegisterEntity);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result removeDevice(String ids) {
        String[] idArray = ids.split(",");
        List deviceCodeList = new ArrayList();
        for(int a=0;a<idArray.length;a++){
            EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getById(Integer.parseInt(idArray[a]));
            if(deviceRegisterEntity == null){
                //暂不确定这里怎么写
                //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            }else{
                if(deviceRegisterEntity.getDeviceStatus()==2){
                    deviceRegisterRepo.delete(deviceRegisterEntity);
                }else{
                    deviceCodeList.add(deviceRegisterEntity.getDeviceCode());
                }
            }
        }
        if(deviceCodeList.size()==0){
            return new Result(ResultCode.SUCCESS);
        }else{
            return new Result(BizConst.CODE_DEVICE_STATUS_EXIST,"这些设备["+deviceCodeList.toString()+"]要先注销再删除.",false);
        }

    }

    //
    private Boolean isExistSameDevice(String deviceCode) {
        int allDeviceSetSize = cacheService.isExistAllDeviceSet();
        if(allDeviceSetSize == 0){
            //{初始化集合}
            initAllDeviceSet();
        }
        boolean isExist = cacheService.isExistDeviceCode(deviceCode);
        return isExist;
    }

    private void initAllDeviceSet() {
        List<Object[]> objectList = deviceRegisterRepo.getDeviceCodeFromDeviceRegister();
        List list = new ArrayList();
        for(int a=0;a<objectList.size();a++){
            Object[] object = (Object[]) objectList.get(a);
            list.add(object[0]);
        }
        System.out.println("list.toString()>>"+list.toString());
        cacheService.putAllDeviceSet(list);
    }
}
